@@ -353,12 +353,39 @@ void
 capplet_set_icon (GtkWidget *window, char const *icon_file_name)
 {
 	char *path;
-	GdkPixbuf *icon_pixbuf;
+	char *tmp;
+	char *p;
+	GdkPixbuf *icon_pixbuf = NULL;
+	GtkIconTheme *icon_theme;
+	GtkIconInfo  *icon_info;
+
+	/* First look up from the icon theme */
+	icon_theme = gtk_icon_theme_get_default ();
+
+	tmp = g_strdup (icon_file_name);
+	p = strrchr (tmp, '.');
+	if (p)
+		p[0] = '\0';
+
+	icon_info = gtk_icon_theme_lookup_icon (icon_theme, tmp, 48, 0);
+
+	if (icon_info != NULL) {
+		icon_pixbuf = gtk_icon_info_load_icon (icon_info, NULL);
+		gtk_icon_info_free (icon_info);
+	}
+
+	g_free (tmp);
 
-	path = g_strconcat (GNOMECC_DATA_DIR "/icons/", icon_file_name, NULL);
-	icon_pixbuf = gdk_pixbuf_new_from_file (path, NULL);
-	g_free (path);
 	if (icon_pixbuf == NULL) {
+		/* Then we fallback to the control center icon location */
+		path = g_strconcat (GNOMECC_DATA_DIR "/icons/", icon_file_name, NULL);
+
+		icon_pixbuf = gdk_pixbuf_new_from_file (path, NULL);
+		g_free (path);
+	}
+
+	if (icon_pixbuf == NULL) {
+		/* Then we fallback to the gnome program discovery stuff */
 		path = gnome_pixmap_file (icon_file_name);
 		if (path != NULL) {
 			icon_pixbuf = gdk_pixbuf_new_from_file (path, NULL);
@@ -367,7 +394,11 @@ capplet_set_icon (GtkWidget *window, char const *icon_file_name)
 	}
 
 	if (icon_pixbuf != NULL) {
+		/* Make sure that every window gets an icon */
+		gtk_window_set_default_icon (icon_pixbuf);
+
 		gtk_window_set_icon (GTK_WINDOW (window), icon_pixbuf);
+
 		g_object_unref (icon_pixbuf);
 	}
 }
