@@ -20,9 +20,9 @@
 #include <string.h>
 #include <X11/keysym.h>
 
+#include <glib/gi18n.h>
 #include <gdk/gdkx.h>
-#include <gnome.h>
-#include <libgnome/gnome-util.h>
+#include <libgnome/gnome-program.h>
 
 #include <libxklavier/xklavier_config.h>
 
